/**
* @made_by : DeathSinger4221
* @version : 1.3.0
* @date : 29.3.2023
*/
import { world } from '@minecraft/server';
import { ActionFormData, ModalFormData } from '@minecraft/server-ui';


world.events.itemUse.subscribe(({ item, source }) => {
    if (item.typeId == 'ftext:floating_text_menu') floatingTextMenu(source);
});


function floatingTextMenu(viewer) {
    const ui = new ActionFormData();
    ui.title('Floating Text Menu');
    ui.button('Create Floating Text', 'textures/ui/recipe_book_expand_icon');
    ui.button('Find Text by Name', 'textures/ui/magnifyingGlass');

    const texts = [];
    for (const egg of viewer.dimension.getEntities({ type: 'minecraft:egg', families: ['inanimate'] })) {
        ui.button(egg.nameTag.replace(/\n[^]+/, ''));
        texts.push(egg);
    }

    ui.show(viewer).then(({ selection }) => {
        if (selection === 0) createText(viewer);
        else if (selection === 1) findText(viewer);
        else editText(viewer, texts[selection - 2]);
    })
};


function createText(viewer) {
    const ui = new ModalFormData();
    ui.title('Create Floating Text');
    ui.textField('Floating Text', 'Text');
    ui.toggle('Centre to Block', false);
    ui.show(viewer).then(({ formValues }) => {
        if (!formValues[0]) { viewer.playSound('note.bass'); createText(viewer); return };
        const { location } = viewer;
        if (formValues[1]) Object.keys(location).forEach(k => location[k] = Math.floor(location[k]) + 0.5);
        const { x, y, z } = location;
        const entity = viewer.dimension.spawnEntity('minecraft:egg<ftext:floating_text>', { x: x, y: y - 0.6, z: z });
        entity.nameTag = formValues[0].replace(/\\n/g, '\n');
    })
};


function findText(viewer, prevError) {
    const ui = new ModalFormData();
    ui.title('Find Text by Name');
    ui.textField(`Search for Floating Text name ${prevError ?? ''}`, 'Text');
    ui.show(viewer).then(({ formValues }) => {
        const ui1 = new ActionFormData();
        ui1.title('Find Text by Name');
        ui1.body(`Here are the texts found by the search for "${formValues[0]}"`);

        const texts = [];
        for (const egg of viewer.dimension.getEntities({ type: 'minecraft:egg', families: ['inanimate'] }))
            if (egg.nameTag.replace(/§.?/g, '').toLocaleLowerCase().includes(formValues[0].replace(/§.?/g, '').trim().toLocaleLowerCase())) {
                ui1.button(egg.nameTag.replace(/\n[^]+/, ''));
                texts.push(egg);
            }

        ui1.button('§cReturn§r', 'textures/ui/arrow_left');

        if (texts.length == 0) {
            viewer.playSound('note.bass');
            findText(viewer, `\n§cNo floating text matched with "${formValues[0]}"`);
            return;
        }
        else ui1.show(viewer).then(({ selection }) => {
            if (selection == texts.length) findText(viewer);
            else editText(viewer, texts[selection]);
        })
    })
};


function editText(viewer, textEntity) {
    const { x, y, z } = textEntity.location;
    const ui = new ModalFormData();
    ui.title('Edit Floating Text');
    ui.toggle('Remove Floating Text', false);
    ui.textField('Floating Text', 'Text', textEntity.nameTag.replace(/\n/g, '\\n'));
    ui.textField('Text Position', 'x y z', `${x.toFixed(2)} ${(y + 0.6).toFixed(2)} ${z.toFixed(2)}`);
    ui.show(viewer).then(({ formValues }) => {
        if (formValues[0]) textEntity.kill();
        else {
            const posArray = formValues[2]?.trim().replace(/ +/g, ' ').split(' ', 3);
            if (!formValues[1]) { viewer.playSound('note.bass'); editText(viewer, textEntity); return };
            textEntity.runCommandAsync(`tp ${posArray[0]} ${posArray[1] - 0.6} ${posArray[2]}`).catch(() => {
                viewer.playSound('note.bass');
                editText(viewer, textEntity);
                return;
            });
            textEntity.nameTag = formValues[1].replace(/\\n/g, '\n');
        }
    })
};
/**
* @made_by : DeathSinger4221
* @version : 1.3.0
* @date : 29.3.2023
*/